/****************************************************************
 *  pic2pac.c
 *			Packprogramm fr Bilder
 *			=======================
 *
 *  31.01.99	Bodo Wenzel	V1.0, bernahme aus HELP-Projekt
 *				und quick-and-dirty erweitert
 ****************************************************************/

/*
Zweck:
- Einlesen von monochromen ST-PICs
- Ausschneiden vom oberen linken Teilbild
- Packen hnlich LZW-Algorithmus

Format:
Prefixbytes fr je 8 Pakete (MSBit nach LSBit):
- 0 = literal = 1 Byte direkt
- 1 = replication = 2 Byte LSByte vor MSByte,
                    Anzahl (4 Bit) und Offset (12 Bit)
*/

#include <stdio.h>

#define  FALSE  (0)
#define  TRUE   (!FALSE)

#define	 TR     (3)
#define	 CBITS  (4)
#define	 CMAX   (1<<CBITS)
#define	 OBITS  (16-CBITS)
#define	 OMAX   (1<<OBITS)
#define	 FMAX   (OMAX+TR-1)

#define  BUF_SIZE  (32000)
#define  PAC_SIZE  (32000)

typedef  unsigned char  uchar;
typedef  unsigned int   uint;

uchar  buffer[BUF_SIZE];
uchar  packed[PAC_SIZE];
long   len_data,len_pac;

uint  ca[0x100],cae[0x100];
struct offs {
  uchar  *o;
  uint   n;
  int    c;
  }  oa[FMAX];

int read_data(char *filename,char *width,char *height);
int dolzw(void);
int save_lzw(char *filename);

int main(int argc,char *argv[]) {
/*-----------------------------*/
if (argc!=5) {
  puts("usage: pic2pac source width height destination");
  return(1);
  }

if (!read_data(argv[1],argv[2],argv[3]))
  return(2);
if (!dolzw())
  return(3);
if (!save_lzw(argv[4]))
  return(4);
return(0);
}

int read_data(char *filename,char *width,char *height) {
/*----------------------------------------------------*/
FILE   *f;
int    w,h,y;
uchar  *p;

if (sscanf(width,"%d",&w)!=1) {
  puts("can't read width!");
  return(FALSE);
  }
if (sscanf(height,"%d",&h)!=1) {
  puts("can't read height!");
  return(FALSE);
  }
f=fopen(filename,"rb");
if (f==NULL) {
  printf("can't open source!");
  return(FALSE);
  }

p=buffer;
for (y=0; y<h; y++) {
  if (p+80>buffer+BUF_SIZE) {
    puts("source too big!");
    return(FALSE);
    }
  if (fread(p,sizeof(char),(size_t)80,f)!=(size_t)80) {
    printf("read error on source!");
    fclose(f);
    return(FALSE);
    }
  p+=w/8;
  }

if (ferror(f)) {
  printf("read error on source!");
  fclose(f);
  return(FALSE);
  }

fclose(f);
len_data=p-buffer;
return(TRUE);
}

int dolzw(void) {
/*-------------*/
register struct offs  *oap;
int                   j,mm,m,mi,ri;
uchar                 *o,*d,*r,p,*b,*c,*mx;
int                   mmx;
uint                  ui;
uchar                 cc;
uint                  f,ii;

for (ii=0; ii<0x100; ii++)
  ca[ii]=FMAX;
for (f=0; f<FMAX; f++)
  oa[(long)f].c=-1;
f=0;

d=packed;
ri=0;
r=d++;
p=0x00;
mx=buffer+len_data;
o=buffer;
while (o<mx) {
  m=TR-1;

  cc=*o;
  ii=ca[cc];
  while (ii<FMAX) {
    oap=&oa[(long)ii];
    b=oap->o;
    c=o;
    j=(int)(c-b);
    if (j<=m)
      break;
    b++;
    c++;
    if (*b++==*c++ && *b++==*c++ && c<=mx && j>=TR) {
      mm=TR-1;
      mmx=j;
      if (mmx>CMAX-1+TR)
        mmx=CMAX-1+TR;
      do {
        mm++;
        if (c>=mx || mm>=mmx)
          break;
      } while (*b++==*c++);
      if (mm>m) {
        mi=j;
        m=mm;
        if (m==CMAX-1+TR)
          break;
        }
      }
    ii=oap->n;
    }

  p<<=1;
  if (m>=TR) {
    p|=0x01;
    ui=((m-TR)<<OBITS) | (mi-TR);
    *d++=(uchar)ui;
    *d++=(uchar)(ui>>8);
  } else {
    *d++=*o;
    m=1;
    }

  do {
    cc=*o;
    if (ca[cc]<FMAX)
      oa[(long)cae[cc]].n=f;
    else
      ca[cc]=f;
    cae[cc]=f;
    oap=&oa[(long)f];
    j=oap->c;
    if (j>=0)
      ca[j]=oa[(long)ca[j]].n;
    oap->c=cc;
    oap->o=o++;
    oap->n=FMAX;
    f=(f+1)%FMAX;
  } while (--m>0);

  if (++ri==8) {
    ri=0;
    *r=p;
    r=d++;
    p=0x00;
    }
  if (d>packed+PAC_SIZE) {
    puts("buffer overrun!");
    return(FALSE);
    }
  }

if (ri!=0) {
  p<<=8-ri;
  *r=p;
} else
  d--;

len_pac=d-packed;
return(TRUE);
}

int save_lzw(char *filename) {
/*--------------------------*/
FILE  *f;

f=fopen(filename,"wb");
if (f==NULL) {
  puts("can't open destination!");
  return(FALSE);
  }

fwrite(packed,sizeof(uchar),(size_t)len_pac,f);

if (ferror(f)) {
  printf("write error on destination!");
  fclose(f);
  return(FALSE);
  }

fclose(f);
return(TRUE);
}

/* ende */
