/****************************************************************
 *  zx97boot.c
 *		Zusammenkopieren von FPGA- und Z80-Binaries
 *		===========================================
 *
 *  24.02.99	Bodo Wenzel	V1.0, von zx97copy bernommen
 ****************************************************************/

/* Vor den eigentlichen FPGA-Daten knnen weitere Bytes kommen,
   die aber nicht das Bitmuster 0100 enthalten drfen (auch ber
   Bytegrenzen (z. B. 00xxxxxx xxxxxx01). Die Anzahl Bits in den
   FPGA-Daten mu dann entsprechend angepat werden.
*/

#include <stdio.h>

#define  FPGA_SIZE   (0xF0E)
#define  JUMP_SIZE   (0x69)	/* NMI+3 Bytes fr Sprung	*/
#define  COPY_START  (FPGA_SIZE+JUMP_SIZE)
#define  RES_START   (0xF77)	/* keine einzelnen 1-Bits!	*/
#define  NMI_START   (0xF7C)	/* keine einzelnen 1-Bits!	*/

#if RES_START < COPYSTART || NMI_START < COPYSTART
#error Bitte *_START anpassen!
#endif

typedef  unsigned char  uchar;
typedef  unsigned int   uint;

#define  BUF_SIZE  (0x2000)
uchar  buffer[BUF_SIZE];

#define  FONT_SIZE  (0x200)

int main(int argc,char *argv[]) {
/*-----------------------------*/
FILE  *f;
int   i;
long  l1,l2;

if (argc!=6) {
  puts("usage: zx97boot fpga-file font-rom z80-binary pic-file"
       " destination");
  return(1);
  }

/* --- Sprnge vorbereiten ------------------------------------ */

buffer[0]=0xC3;			/*	jp	RES_START	*/
buffer[1]=(uchar)(RES_START&0xFF);
buffer[2]=(uchar)(RES_START>>8);
for (i=3; i<JUMP_SIZE-3; i++)
  buffer[i]=0x00;
buffer[JUMP_SIZE-3]=0xC3;	/*	jp	NMI_START	*/
buffer[JUMP_SIZE-2]=(uchar)(NMI_START&0xFF);
buffer[JUMP_SIZE-1]=(uchar)(NMI_START>>8);

/* --- FPGA-Konfiguration holen und patchen ------------------- */

f=fopen(argv[1],"rb");
if (f==NULL) {
  printf("can't open fpga-file!");
  return(2);
  }
fseek(f,0,SEEK_END);
if (ftell(f)!=FPGA_SIZE) {
  printf("wrong size of fpga-file!");
  fclose(f);
  return(2);
  }
fseek(f,0,SEEK_SET);
if (fread(buffer+JUMP_SIZE,
          sizeof(uchar),FPGA_SIZE,f)!=FPGA_SIZE) {
  printf("read error on fpga-file!");
  fclose(f);
  return(2);
  }
fclose(f);

l1=((long)buffer[JUMP_SIZE+4]<<24)+
   ((long)buffer[JUMP_SIZE+3]<<16)+
   ((long)buffer[JUMP_SIZE+2]<<8)+
   ((long)buffer[JUMP_SIZE+1]<<0);
l2=0;
for (i=0; i<24; i++) {
  l2<<=1;
  if (l1&0x10)
    l2++;
  l1>>=1;
  }
l2+=JUMP_SIZE*8;
l1=0xF;
for (i=0; i<24; i++) {
  l1<<=1;
  if (l2&0x1)
    l1++;
  l2>>=1;
  }
l1<<=4;
l1|=0x4;
buffer[JUMP_SIZE+4]=(uchar)(l1>>24);
buffer[JUMP_SIZE+3]=(uchar)(l1>>16);
buffer[JUMP_SIZE+2]=(uchar)(l1>>8);
buffer[JUMP_SIZE+1]=(uchar)(l1>>0);

/* --- Font holen --------------------------------------------- */

f=fopen(argv[2],"rb");
if (f==NULL) {
  printf("can't open font-rom!");
  return(3);
  }
fseek(f,0,SEEK_END);
l1=ftell(f);
if (l1<FONT_SIZE) {
  printf("font-rom too small!");
  fclose(f);
  return(3);
  }
fseek(f,-FONT_SIZE,SEEK_END);
if (fread(buffer+BUF_SIZE-FONT_SIZE,
          sizeof(uchar),FONT_SIZE,f)!=FONT_SIZE) {
  printf("read error on font-rom!");
  fclose(f);
  return(3);
  }
fclose(f);

/* --- Z80-Code holen ----------------------------------------- */

f=fopen(argv[3],"rb");
if (f==NULL) {
  printf("can't open z80-binary!");
  return(4);
  }
fseek(f,0,SEEK_END);
l1=ftell(f)-JUMP_SIZE-FPGA_SIZE;
if (l1<0) {
  printf("z80-binary too small!");
  fclose(f);
  return(4);
  }
if (l1>BUF_SIZE-JUMP_SIZE-FPGA_SIZE-FONT_SIZE) {
  printf("z80-binary too big!");
  fclose(f);
  return(4);
  }
fseek(f,JUMP_SIZE+FPGA_SIZE,SEEK_SET);
if (fread(buffer+JUMP_SIZE+FPGA_SIZE,
          sizeof(uchar),l1,f)!=l1) {
  printf("read error on z80-binary!");
  fclose(f);
  return(4);
  }
fclose(f);

/* --- Bild holen --------------------------------------------- */

f=fopen(argv[4],"rb");
if (f==NULL) {
  printf("can't open pic-file!");
  return(5);
  }
fseek(f,0,SEEK_END);
l2=ftell(f);
if (l2>BUF_SIZE-JUMP_SIZE-FPGA_SIZE-l1-FONT_SIZE) {
  printf("pic-file too big!");
  fclose(f);
  return(5);
  }
fseek(f,0,SEEK_SET);
if (fread(buffer+JUMP_SIZE+FPGA_SIZE+l1,
          sizeof(uchar),l2,f)!=l2) {
  printf("read error on pic-file!");
  fclose(f);
  return(5);
  }
fclose(f);

/* --- Rest fr EPROMs fllen --------------------------------- */

for (i=(int)(JUMP_SIZE+FPGA_SIZE+l1+l2);
     i<BUF_SIZE-FONT_SIZE; i++)
  buffer[i]=0xFF;

/* --- Komplettes Image sichern ------------------------------- */

f=fopen(argv[5],"wb");
if (f==NULL) {
  puts("can't open destination!");
  return(6);
  }
fwrite(buffer,sizeof(uchar),BUF_SIZE,f);
if (ferror(f)) {
  printf("write error on destination!");
  fclose(f);
  return(6);
  }
fclose(f);

return(0);
}

/* --- Ende --------------------------------------------------- */
